@@ -44,7 +44,6 @@
 #include "nodes/makefuncs.h"
 #include "optimizer/clauses.h"
 #include "optimizer/planner.h"
-#include "optimizer/var.h"
 #include "parser/analyze.h"
 #include "parser/parse_node.h"
 #include "parser/parsetree.h"
@@ -851,7 +850,8 @@ pgqs_queryentry_dealloc(void)
 	entry = hash_seq_search(&hash_seq);
 
 	if (entry != NULL)
-		hash_search_with_hash_value(pgqs_query_examples_hash, &entry->key, ((pgqsQueryStringHashKey) entry->key).queryid, HASH_REMOVE, NULL);
+		hash_search_with_hash_value(pgqs_query_examples_hash, &entry->key,
+				entry->key.queryid, HASH_REMOVE, NULL);
 
 	hash_seq_term(&hash_seq);
 }
@@ -1064,7 +1064,7 @@ pgqs_process_scalararrayopexpr(ScalarArrayOpExpr *expr, pgqsWalkerContext *conte
 {
 	OpExpr	   *op = makeNode(OpExpr);
 	int			len = 0;
-	pgqsEntry  *entry;
+	pgqsEntry  *entry = NULL;
 	Expr	   *array = lsecond(expr->args);
 
 	op->opno = expr->opno;
@@ -1095,11 +1095,13 @@ pgqs_process_scalararrayopexpr(ScalarArrayOpExpr *expr, pgqsWalkerContext *conte
 		default:
 			break;
 	}
+
 	if (len > 0)
 	{
 		context->count *= len;
 		entry = pgqs_process_opexpr(op, context);
 	}
+
 	return entry;
 }
 
@@ -1462,6 +1464,9 @@ pgqs_process_opexpr(OpExpr *expr, pgqsWalkerContext *context)
 											  &found);
 			if (!found)
 			{
+				char   *utf8const;
+				int		len;
+
 				context->nentries++;
 
 				memcpy(&(entry->lrelid), &(tempentry.lrelid), sizeof(pgqsEntry) - sizeof(pgqsHashKey));
@@ -1472,11 +1477,24 @@ pgqs_process_opexpr(OpExpr *expr, pgqsWalkerContext *context)
 				entry->position = position;
 				entry->qualnodeid = hashExpr((Expr *) expr, context, false);
 				entry->qualid = context->qualid;
-				strncpy(entry->constvalue, (char *) pg_do_encoding_conversion((unsigned char *) buf->data,
-																			  strlen(buf->data),
-																			  GetDatabaseEncoding(),
-																			  PG_UTF8),
-						PGQS_CONSTANT_SIZE);
+
+				utf8const = (char *) pg_do_encoding_conversion((unsigned char *) buf->data,
+																strlen(buf->data),
+																GetDatabaseEncoding(),
+																PG_UTF8);
+				len = strlen(utf8const);
+
+				/*
+				 * The const value can use multibyte characters, so we need to
+				 * be careful when truncating the value.  Note that we need to
+				 * use PG_UTF8 encoding explicitly here, as the value was just
+				 * converted to this encoding.
+				 */
+				len = pg_encoding_mbcliplen(PG_UTF8, utf8const, len,
+						PGQS_CONSTANT_SIZE - 1);
+
+				memcpy(entry->constvalue, utf8const, len);
+				entry->constvalue[len] = '\0';
 
 				if (pgqs_resolve_oids)
 					pgqs_fillnames((pgqsEntryWithNames *) entry);
